#include "key.h"
/*
 按键检测。
*/
// 当前设计，最多8个按键，因为使用了uint8_t类型来存储。
#define MAX_KEY_NUM 2



/* Key state - shift or nornal */
#define HAL_KEY_STATE_NORMAL          0x00
#define HAL_KEY_STATE_SHIFT           0x01

typedef void (*halKeyCBack_t) (uint8_t keys, uint8_t state);

static uint8_t halKeySavedKeys;     /* used to store previous key state in polling mode */

static halKeyCBack_t pHalKeyProcessFunction;

// 按键初始化
void HalKeyInit(void *cback){
  // 输入上拉，低电平触发按键。
  pinMode(PIN_BTN_S1, INPUT_PULLUP);
  pinMode(PIN_BTN_S2, INPUT_PULLUP);
  
  // arduino的形参不支持自定义的typedef，奇怪。
  pHalKeyProcessFunction = (halKeyCBack_t)cback;
}

// 读按键状态。按位赋值，最多支持8个按键。
uint8_t HalKeyRead(){
  uint8_t key = 0;
  if( digitalRead(PIN_BTN_S1) == LOW){
    key |= HAL_KEY_BUTTON_S1;
  }
  if( digitalRead(PIN_BTN_S2) == LOW){
    key |= HAL_KEY_BUTTON_S2;
  }
  return key;
}


// 带消抖版本。
void HalKeyPoll (void){
  uint8_t keys = 0;
  uint8_t notify = 0;
  static bool delayRead = false;
  static unsigned long time = 0;
  keys = HalKeyRead();
  if (keys == halKeySavedKeys && delayRead == false) {
    // Exit - since no keys have changed
    return;
  } else {
    // notify = 1;
  }

  // 第一次触发。保存时间。若是第二次，则delayRead=true
  if(delayRead == false){
    // 记录这一次的按键。
    halKeySavedKeys = keys;
    time = millis();
    delayRead = true;
    return ;
  }
  // 去抖动，准备第二次读取数据。
  if((millis() - time) > 80){
    if(halKeySavedKeys == keys){
      // 延时后，重新检测和上次的结果相同。则
      notify = 1;
    }else {
      // 重新开始
      delayRead = false;
      halKeySavedKeys = 0;
    }
  }
  // Invoke Callback if new keys were depressed
  if (notify && (pHalKeyProcessFunction))
  {
    static uint8_t lastKeys = 0;
    delayRead = false;
    // 按键按下时，触发 keys & KEY1 == true && lastKeys & KEY1 == false 事件
    // 按键释放时，触发 keys & KEY1 == false && lastKeys & KEY1 == true 事件
    (pHalKeyProcessFunction) (keys, lastKeys);
    lastKeys = keys;
  }
}

// 判断按键按下还是释放，如果释放，time是按下和释放的时间差。单位为ms。
uint8_t KeyPressed(uint8_t keys, uint8_t lastKeys, uint8_t keyId, unsigned long *time){
  static unsigned long pressedTime[MAX_KEY_NUM] = {0,0};
  uint8_t keyIndex = getKeyIndex(keyId);
  if((keys & keyId) && !(lastKeys & keyId)){
    pressedTime[keyIndex] = millis();
    return KEY_PRESSED;
  }else if((lastKeys & keyId) && !(keys & keyId)){
    if(time){
      *time = millis() - pressedTime[keyIndex];
    }
    return KEY_RELEASED;
  }else{
    return KEY_NONE;
  }
}
// 按键id转index，index从0开始。
uint8_t getKeyIndex(uint8_t keyId){
  for(int i = 1; i<= MAX_KEY_NUM; i++){
    if(bitRead(keyId,i)){
      return i-1;
    }
  }
}

// 指定按键引脚的高低电平
uint8_t getKeyStatus(uint8_t key)
{
  return digitalRead(key);
}
